<?php

namespace Drupal\smallads\Plugin\Field\FieldType;

use Drupal\smallads\Entity\SmalladInterface;
use Drupal\Core\TypedData\OptionsProviderInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\IntegerItem;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Field\FieldDefinitionInterface;

/**
 * Defines the 'scope' smallad field type.
 *
 * @FieldType(
 *   id = "smallad_scope",
 *   label = @Translation("Smallad Scope"),
 *   description = @Translation("How widely this ad can be seen"),
 *   category = @Translation("Smallads"),
 *   default_widget = "options_select"
 * )
 */


class ScopeItem extends IntegerItem implements OptionsProviderInterface{

  /**
   * {@inheritdoc}
   */
  public function getPossibleValues(AccountInterface $account = NULL) {
    return array_keys($this->getPossibleOptions($account));
  }

  /**
   * {@inheritdoc}
   */
  public function getPossibleOptions(AccountInterface $account = NULL) {
    $scopes = [
      SmalladInterface::SCOPE_PRIVATE => t('Just you'),
      SmalladInterface::SCOPE_SITE => t('Members of this site'),
      SmalladInterface::SCOPE_PUBLIC => t('Public'),
    ];
    if (\Drupal::moduleHandler()->moduleExists('group')) {
      $scopes[SmalladInterface::SCOPE_GROUP] = t('Your groups');
    }
    // @todo use widget_alter?
    if (\Drupal::moduleHandler()->moduleExists('smallads_index')) {
      $scopes[SmalladInterface::SCOPE_NETWORK] = t('The Solsearch network');
    }
    ksort($scopes);
    return $scopes;
  }

  /**
   * {@inheritdoc}
   */
  public function getSettableValues(AccountInterface $account = NULL) {
    return array_keys($this->getPossibleValues($account));
  }

  /**
   * {@inheritdoc}
   */
  public function getSettableOptions(AccountInterface $account = NULL) {
    return $this->getPossibleOptions($account);
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition) {
    return rand(SmalladInterface::SCOPE_PRIVATE, SmalladInterface::SCOPE_PUBLIC);
  }


  /**
   *
   * Entity field default callback for scope field.
   * The default scope for a smallad is based on the settings for its owner.
   *
   * @todo make this a site setting
   *
   * @param SmalladInterface $smallad
   * @param \Drupal\smallads\Plugin\Field\FieldType\BaseFieldDefinition $definition
   *
   * @return type
   */
  public static function smalladsDefaultScope(SmalladInterface $smallad) {
    return \Drupal::service('user.data')
      ->get('smallads', $smallad->getOwnerId(), 'defaultscope') ?: SmalladInterface::SCOPE_SITE;
  }

}
